#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content_es.interfaces import IContentIndexerUtility, IDocumentIndexTarget
from transaction.interfaces import ITransactionManager
from zope.lifecycleevent.interfaces import IObjectAddedEvent, IObjectModifiedEvent, IObjectRemovedEvent

# import packages
from pyams_utils.registry import query_utility
from pyramid.events import subscriber


#
# Documents events
#

def index_document(status, document):
    if not status:  # aborted transaction
        return
    indexer = query_utility(IContentIndexerUtility)
    if indexer is not None:
        indexer.index_document(document)


def unindex_document(status, document):
    if not status:  # aborted transaction
        return
    indexer = query_utility(IContentIndexerUtility)
    if indexer is not None:
        indexer.unindex_document(document)


@subscriber(IObjectAddedEvent, context_selector=IDocumentIndexTarget)
def handle_added_document(event):
    """Handle added document"""
    document = event.object
    ITransactionManager(document).get().addAfterCommitHook(index_document, kws={'document': document})


@subscriber(IObjectModifiedEvent, context_selector=IDocumentIndexTarget)
def handle_modified_document(event):
    """Handle modified document"""
    document = event.object
    transaction = ITransactionManager(document).get()
    if 'pyams_content_es.index_document' not in transaction.extension:
        transaction.addAfterCommitHook(index_document, kws={'document': document})
        transaction.extension['pyams_content_es.index_document'] = True


@subscriber(IObjectRemovedEvent, context_selector=IDocumentIndexTarget)
def handle_removed_document(event):
    """Handle removed document"""
    document = event.object
    ITransactionManager(document).get().addAfterCommitHook(unindex_document, kws={'document': document})
